/*
    Magic eye handling functions.
    Copyright (C) 2017  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file magiceye.h
     \author Alessandro Accattini
     \date 24 Apr 2017
     \brief Magic eye handling functions header file.

     A magic eye is a circular projection of a rectangular pixels array,
     setting the color of a pixel in the rectangular array is equivalent to
     setting the color of a little circular sector of the eye.

     This file provides all functions to create and handle one or more magic
     eyes. A magic eye is based on a pre-built data set, but you need to
     create an SDL_Surface wich will contain the eye itself, in fact calling
     MagicEye_Perspection() means to update the SDL_Surface containing the
     magic eye; subsequent scene redraw is required.
 */


#ifndef __MAGICEYE_H
#define __MAGICEYE_H	1


#include <SDL/SDL.h>				// SDL library data types

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* macro pubbliche */

/***************************************************************************/
/* strutture pubbliche */

/** \brief Color data type.
 *
 */
typedef unsigned short int	Color_t;


/** \brief Raster descriptor structure.
 *
 */
typedef struct
{
  int		Lines;		/**< Number of raster lines. */
  const void	*Raster;	/**< The raster lines table. */
}RasterDesc_t;


/** \brief Magic eye descriptor structure.
 *
 */
typedef struct
{
  Uint16		w,h;	/**< SDL_Surface width and height. */
  Uint16		u,v;	/**< The width and height of linear space. */
  const RasterDesc_t	*Raster;/**< The raster descriptors table, this table
  				     has many entries as v size indicates. */
  Color_t		*Pixel;	/**< Pixel linear array pointer. */
}MagicEye_t;

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

SDL_Surface *MagicEye_CreateSurface(const MagicEye_t *EyeDesc);
  /**< \brief  creates the required magic eye surface.
       \param EyeDesc  the magic eye descriptor.
       \return  the new SDL_Surface

       You need an SDL_Surface in wich the magic eye is created and updated,
       this is the first function you have to call in order to create a magic
       eye.
   */

void MagicEye_PutPixel(const MagicEye_t *EyeDesc,int u,int v,Color_t Color);
  /**< \brief  place a pixel in the rectangular array.
       \param EyeDesc  the magic eye descriptor.
       \param u  the horizontal coordinate in the rectangular array, this
       		 coordinate is equivalent to the agular coordinate in the eye.
       \param v  the vertical coordinate in the rectangular array, this
       		 coordinate is oriented radially in the eye, starting from the
		 external ring toward to the centre of the eye.
       \param Color  the pixel color.

       This function simply place a pixel in the rectangular array, no
       changes to the magic eye are implied, to update the SDL_Surface you
       need to explicitely call MagicEye_Perspection().
   */

Color_t MagicEye_GetPixel(const MagicEye_t *EyeDesc,int u,int v);
  /**< \brief  reads back the pixel color.
       \param EyeDesc  the magic eye descriptor.
       \param u  the horizontal coordinate in the rectangular array, this
       		 coordinate is equivalent to the agular coordinate in the eye.
       \param v  the vertical coordinate in the rectangular array, this
       		 coordinate is oriented radially in the eye, starting from the
		 external ring toward to the centre of the eye.
       \return  the pixel color.

       This function can be used to read back a pixel color (see
       MagicEye_PutPixel()).
   */

void MagicEye_Fill(const MagicEye_t *EyeDesc,Color_t Color);
  /**< \brief  fills the whole rectangular array with the same color.
       \param EyeDesc  the magic eye descriptor.
       \param Color  the color to fill with.

       Fills the whole rectangular array at once, usefull for example to
       clear all pixel in a single call.
   */

void MagicEye_Perspection(const MagicEye_t *EyeDesc,int Rotate[],
			  SDL_Surface *Img);
  /**< \brief  updates the magic eye surface.
       \param EyeDesc  the magic eye descriptor.
       \param Rotate  a rotation steps array.
       \param Img  the SDL_Surface in witch the magic eye is drawn.

       This function creates the real magic eye: after all rectangular array
       draw you need to call this function to update the SDL_Surface.
       It is possible to pass to this function one rotation descriptor array:
       Rotate must have one integer for each ring inside the magic eye, this
       is a way to rotate the magic eye rings without touching the linear
       vector. A positive value means to rotate the right in clockwise
       direction, a negative number means to rotate the right anticlockwise;
       if not required Rotate can be NULL.
   */

/***************************************************************************/
#endif

/* $Source$ */

